#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library
from html import escape

# import interfaces
from pyams_skin.interfaces import IInnerPage, IPageHeader, IContentHelp
from pyams_skin.interfaces.container import ITableWithActions
from pyams_skin.layer import IPyAMSLayer
from pyams_utils.interfaces import MANAGE_SYSTEM_PERMISSION
from pyams_utils.interfaces.site import IOptionalUtility
from pyams_zmi.interfaces.menu import IControlPanelMenu, IUtilitiesMenu
from pyams_zmi.layer import IAdminLayer
from z3c.table.interfaces import IValues, IColumn
from zope.component.interfaces import ISite

# import packages
from pyams_pagelet.pagelet import pagelet_config
from pyams_skin.container import ContainerView
from pyams_skin.help import ContentHelp
from pyams_skin.page import DefaultPageHeaderAdapter
from pyams_skin.table import BaseTable, TrashColumn
from pyams_skin.viewlet.menu import MenuItem
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config
from pyams_utils.registry import registered_utilities
from pyams_utils.text import text_to_html
from pyams_utils.url import absolute_url
from pyams_viewlet.manager import viewletmanager_config
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.view import AdminView
from pyramid.view import view_config
from z3c.table.column import GetAttrColumn
from zope.interface import implementer, Interface

from pyams_zmi import _


#
# Utilities views and adapters
#

@viewlet_config(name='utilities.menu', layer=IAdminLayer, context=ISite, manager=IControlPanelMenu,
                permission=MANAGE_SYSTEM_PERMISSION, weight=1)
@viewletmanager_config(name='utilities.menu', layer=IAdminLayer, context=ISite, provides=IUtilitiesMenu)
@implementer(IUtilitiesMenu)
class UtilitiesMenuItem(MenuItem):
    """Utilities menu"""

    label = _("Utilities")
    icon_class = 'fa-codepen'
    url = '#utilities.html'


@implementer(ITableWithActions)
class UtilitiesTable(BaseTable):
    """Utilities table"""

    id = 'utilities_table'
    title = _("Site utilities")

    @property
    def data_attributes(self):
        attributes = super(UtilitiesTable, self).data_attributes
        attributes['table'] = {'data-ams-location': absolute_url(self.context, self.request),
                               'data-ams-delete-target': 'delete-utility.json'}
        return attributes


@adapter_config(name='trash', context=(Interface, IAdminLayer, UtilitiesTable), provides=IColumn)
class UtilitiesTrashColumn(TrashColumn):
    """Utilities trash column"""

    icon_hint = _("Delete utility")
    permission = MANAGE_SYSTEM_PERMISSION

    checker = lambda col, x: IOptionalUtility.providedBy(x)


@adapter_config(context=(ISite, IAdminLayer, UtilitiesTable), provides=IValues)
class UtilitiesValuesAdapter(ContextRequestViewAdapter):
    """Utilities values adapter"""

    @property
    def values(self):
        return list(self.context.getSiteManager().values())


@pagelet_config(name='utilities.html', context=ISite, layer=IPyAMSLayer, permission=MANAGE_SYSTEM_PERMISSION)
@implementer(IInnerPage)
class UtilitiesView(AdminView, ContainerView):
    """Control panel view"""

    table_class = UtilitiesTable

    def __init__(self, context, request):
        super(UtilitiesView, self).__init__(context, request)


@adapter_config(context=(ISite, IAdminLayer, UtilitiesView), provides=IPageHeader)
class UtilitiesHeaderAdapter(DefaultPageHeaderAdapter):
    """Utilities header adapter"""

    icon_class = 'fa fa-fw fa-codepen'
    title = _("Control panel")


@view_config(name='delete-utility.json', context=ISite, request_type=IPyAMSLayer,
             permission=MANAGE_SYSTEM_PERMISSION, renderer='json', xhr=True)
def delete_utility(request):
    """Delete utility from site manager"""
    translate = request.localizer.translate
    name = request.params.get('object_name')
    if not name:
        return {'status': 'message',
                'messagebox': {'status': 'error',
                               'content': translate(_("No provided object_name argument!"))}}
    manager = request.context.getSiteManager()
    if name not in manager:
        return {'status': 'message',
                'messagebox': {'status': 'error',
                               'content': translate(_("Given utility name doesn't exist!"))}}
    del manager[name]
    return {'status': 'success'}


#
# Common registrations views and adapters
#

class IRegistrationsTable(Interface):
    """Registrations view marker interface"""


@adapter_config(name='component', context=(ISite, IAdminLayer, IRegistrationsTable), provides=IColumn)
class RegistrationsComponentColumn(GetAttrColumn):
    """Registrations component column"""

    _header = _("Component")
    weight = 1

    @property
    def header(self):
        return self.request.localizer.translate(self._header)

    def getValue(self, obj):
        component = obj.component
        if component is not None:
            name = getattr(component, '__name__', None)
            if not name:
                name = str(component.__class__)
        else:
            name = str(obj.factory)
        return escape(name)


@adapter_config(name='interface', context=(ISite, IAdminLayer, IRegistrationsTable), provides=IColumn)
class RegistrationsInterfaceColumn(GetAttrColumn):
    """Registrations interface column"""

    _header = _("Registered interface")
    weight = 5

    @property
    def header(self):
        return self.request.localizer.translate(self._header)

    def getValue(self, obj):
        return text_to_html(str(obj.provided))


@adapter_config(name='name', context=(ISite, IAdminLayer, IRegistrationsTable), provides=IColumn)
class RegistrationsNameColumn(GetAttrColumn):
    """Registrations name column"""

    _header = _("Name")
    weight = 10

    @property
    def header(self):
        return self.request.localizer.translate(self._header)

    def getValue(self, obj):
        return obj.name or _('< no name >')


#
# Local registrations views
#

@viewlet_config(name='local-registrations.menu', layer=IAdminLayer, context=ISite, manager=IUtilitiesMenu,
                permission=MANAGE_SYSTEM_PERMISSION, weight=1)
class LocalRegistrationsMenuItem(MenuItem):
    """Local registrations menu"""

    label = _("Local registrations")
    url = '#local-registrations.html'


@implementer(IRegistrationsTable)
class LocalRegistrationsTable(BaseTable):
    """Local utilities registrations table"""

    id = 'local_registrations_table'
    title = _("Local utilities registrations")

    data_attributes = {}


@adapter_config(context=(ISite, IAdminLayer, LocalRegistrationsTable), provides=IValues)
class LocalRegistrationsValuesAdapter(ContextRequestViewAdapter):
    """Local utilities values adapter"""

    @property
    def values(self):
        return list(self.context.getSiteManager().registeredUtilities())


@pagelet_config(name='local-registrations.html', context=ISite, layer=IPyAMSLayer,
                permission=MANAGE_SYSTEM_PERMISSION)
@implementer(IInnerPage)
class LocalRegistrationsView(AdminView, ContainerView):
    """Registrations view"""

    table_class = LocalRegistrationsTable

    def __init__(self, context, request):
        super(LocalRegistrationsView, self).__init__(context, request)


@adapter_config(context=(ISite, IAdminLayer, LocalRegistrationsView), provides=IPageHeader)
class LocalRegistrationsHeaderAdapter(DefaultPageHeaderAdapter):
    """Local registrations header adapter"""

    icon_class = 'fa fa-fw fa-codepen'
    title = _("Control panel")


@adapter_config(context=(ISite, IAdminLayer, LocalRegistrationsView), provides=IContentHelp)
class LocalRegistrationsHelpAdapter(ContentHelp):
    """Local registrations help adapter"""

    header = _("Local registry utilities")
    message = _("""A local registry is a registry defining utilities stored into
site's Object Database (ZODB).

You can manage these utilities and modify their properties and site's behaviour without
modifying the application.""")
    message_format = 'rest'


#
# Global registrations views
#

@viewlet_config(name='global-registrations.menu', layer=IAdminLayer, context=ISite, manager=IUtilitiesMenu,
                permission=MANAGE_SYSTEM_PERMISSION, weight=2)
class GlobalRegistrationsMenuItem(MenuItem):
    """Global registrations menu"""

    label = _("Global registrations")
    url = '#global-registrations.html'


@implementer(IRegistrationsTable)
class GlobalRegistrationsTable(BaseTable):
    """Utilities global registrations table"""

    id = 'global_registrations_table'
    title = _("Global utilities registrations")

    data_attributes = {}


@adapter_config(context=(ISite, IAdminLayer, GlobalRegistrationsTable), provides=IValues)
class GlobalRegistrationsValuesAdapter(ContextRequestViewAdapter):
    """Global utilities values adapter"""

    @property
    def values(self):
        return list(registered_utilities())


@pagelet_config(name='global-registrations.html', context=ISite, layer=IPyAMSLayer,
                permission=MANAGE_SYSTEM_PERMISSION)
@implementer(IInnerPage)
class GlobalRegistrationsView(AdminView, ContainerView):
    """Global registrations view"""

    table_class = GlobalRegistrationsTable

    def __init__(self, context, request):
        super(GlobalRegistrationsView, self).__init__(context, request)


@adapter_config(context=(ISite, IAdminLayer, GlobalRegistrationsView), provides=IPageHeader)
class GlobalRegistrationsHeaderAdapter(DefaultPageHeaderAdapter):
    """Global registrations header adapter"""

    icon_class = 'fa fa-fw fa-codepen'
    title = _("Control panel")


@adapter_config(context=(ISite, IAdminLayer, GlobalRegistrationsView), provides=IContentHelp)
class GlobalRegistrationsHelpAdapter(ContentHelp):
    """Global registrations help adapter"""

    header = _("Global registry utilities")
    message = _("""The global registry groups local utilities as well as utilities registered outside
site's Object Database (ZODB).

These utilities are declared statically (generally using include or ZCML directives) and can't be setup
without modifying site configuration.""")
    message_format = 'rest'
